// (c) 2000 Applegame Software Lab
// The given product corresponds to the standard "AS IS", it means, that we
// do not give any warranties concerning operation of the product, and also
// we do not carry the responsibility for the caused losses and/or the missed
// benefit as a result of product usage.
//---------------------------------------------------------------------------

#ifndef ASLanMsgH
#define ASLanMsgH
//---------------------------------------------------------------------------
#include <SysUtils.hpp>
#include <Controls.hpp>
#include <Classes.hpp>
#include <Forms.hpp>
//---------------------------------------------------------------------------

enum TASWorkMode {WMFull,WMHalf,WMLow};

class TASUserInfo : public TObject
{
	friend class TASLanMsg;
private:
	AnsiString FComputerName;
	AnsiString FUserName;
	AnsiString FClientName;
	int FMessageInfo;
public:
	__property AnsiString ComputerName  = { read=FComputerName };
	__property AnsiString UserName  = { read=FUserName };
	__property AnsiString ClientName  = { read=FClientName };
	__property int MessageInfo  = { read=FMessageInfo, write=FMessageInfo };
};

typedef void __fastcall (__closure *TUserStatusEvent)(TObject *Sender,TASUserInfo *From);
typedef void __fastcall (__closure *TDataReceivedEvent)(TObject *Sender,TASUserInfo *From,void* Buf,int Len);

class PACKAGE TASLanMsg : public TComponent
{
    enum NotifyCommand {ONLINE_NOTIFY,
                        DATASEND_NOTIFY,
                        CONNECTED_NOTIFY,
                        DISCONNECTED_NOTIFY,
                        NAMECHANGE_NOTIFY};
    struct Notify
    {
        int MessageID;
        int MessageInfo;
        NotifyCommand Command;
        int CNameLen;
        int UNameLen;
        int CLNameLen;
    };
private:
    TTimer  *RTimer;
    bool FActive;
    HANDLE MSHandle;
    AnsiString FComputer;
    AnsiString FUser;
    AnsiString FSlotName;
    int MessageID;
    TASWorkMode FMode;
    TUserStatusEvent FOnUserOnLine;
    TUserStatusEvent FOnUserDisconnected;
    TUserStatusEvent FOnUserConnected;
    TDataReceivedEvent FOnDataReceived;
    bool Created;
	AnsiString FClientName;
    void __fastcall SetActive(bool value);
    void __fastcall SetSlotName(AnsiString value);
    void __fastcall PSendData(NotifyCommand Command,AnsiString CName,void * Buf=NULL, int Len=0);
    void __fastcall SetMode(TASWorkMode value);
    int __fastcall GetModeValue(TASWorkMode);
    void __fastcall OnTimerEvent(TObject*);
    void __fastcall Activate();
    void __fastcall Deactivate();
	void __fastcall SetClientName(AnsiString value);
	TUserStatusEvent FOnUserChangeName;
	int FMessageInfo;
protected:
    virtual void __fastcall Loaded(void);
public:
    __fastcall TASLanMsg(TComponent* Owner);
    virtual __fastcall ~TASLanMsg();
	void __fastcall Broadcast(void * Buffer, int Len);
	void __fastcall Send(AnsiString Computer, void * Buffer=NULL, int Len=0);
    __property AnsiString Computer  = { read=FComputer };
    __property AnsiString User  = { read=FUser };
__published:
    __property bool Active  = { read=FActive, write=SetActive };
    __property AnsiString SlotName  = { read=FSlotName, write=SetSlotName };
    __property TASWorkMode Mode  = { read=FMode, write=SetMode, default=WMFull};
    __property TUserStatusEvent OnUserOnLine  = { read=FOnUserOnLine, write=FOnUserOnLine };
    __property TUserStatusEvent OnUserDisconnected  = { read=FOnUserDisconnected, write=FOnUserDisconnected };
	__property TUserStatusEvent OnUserChangeName  = { read=FOnUserChangeName, write=FOnUserChangeName };
    __property TDataReceivedEvent OnDataReceived  = { read=FOnDataReceived, write=FOnDataReceived };
    __property TUserStatusEvent OnUserConnected  = { read=FOnUserConnected, write=FOnUserConnected };
	__property AnsiString ClientName  = { read=FClientName, write=SetClientName };
	__property int MessageInfo  = { read=FMessageInfo, write=FMessageInfo };
};
//---------------------------------------------------------------------------
#endif